#!/bin/sh
LOGFILE=/opt/var/log/dibbler/dhcp6update.log

#  1= Log eingeschaltet, 0= Log deaktiviert
LOG="0" 

# do not change ##########################################################################
if [ -f /opt/var/lib/dibbler/dhcp6update.radvbr0.ip ]
then
	RADIPCACHE="$(cat /opt/var/lib/dibbler/dhcp6update.radvbr0.ip)"
else
	RADIPCACHE="1"
fi

RADIP="$(grep 'prefix' /var/radvd/radvdbr0.conf | sed 's/ *prefix *//' | grep '2003'  | tr -d ' \r\n')"

if [ "$RADIP" == "$RADIPCACHE" ]
then
	if [ "$LOG" == "1" ]
	then
		echo "DHCP6Update: nothing to change..." >> $LOGFILE
	fi
else
	# cache the radbr0 ip
	echo "$RADIP" > /opt/var/lib/dibbler/dhcp6update.radvbr0.ip

	TLANPREFIX="$(echo "$RADIP" | sed 's/:0*/:/g')"
	FULLPREFIX="$(echo "$TLANPREFIX" | sed -r 's~\w\w:\w+:\w+:\w+:\w+/64~~')"
	TLANPREFIX="$(echo "$TLANPREFIX" | sed -r 's~\w+:\w+:\w+:\w+/64~:/64~')"

	# Use only a part of the full /56 to make sure dibbler cannot delegate the prefix the router wishes to use
	TLANPREFIX_FIRST="$(echo "$TLANPREFIX" | sed -r 's~.*(\w)::/64~\1~')"
    
	if [ "$TLANPREFIX_FIRST" == "4" ]
	then
		FULLLANPREFIX="${FULLPREFIX}50::/60"
	else
		FULLLANPREFIX="${FULLPREFIX}40::/60"
	fi

	FULLPREFIX="${FULLPREFIX}00::/56"

	if  [ xtables-multi ip6tables-save | grep -qF "$FULLANPREFIX" != "" ]
	then
		if [ "$LOG" == "1"  ]
		then
			echo 'DHCP6Update: IPv6 /60 FW ok' >> $LOGFILE
		fi
	else
		# clean up firewall
		if [ -f /opt/var/lib/dibbler/undo_ipv6_fw.sh ]
		then
			/opt/var/lib/dibbler/undo_ipv6_fw.sh
		fi	
		
		# make new firewall setup 
		echo > /opt/var/lib/dibbler/undo_ipv6_fw.sh
		chmod -x /opt/var/lib/dibbler/undo_ipv6_fw.sh
		ip6tables -t mangle -A PRE_LAN_SUBNET -s $FULLLANPREFIX -i br0 -j ACCEPT
		ip6tables -t mangle -I ROUTE_CTL_LIST -d $FULLLANPREFIX -j RETURN	
		ip6tables -F FORWARD_PREFIX
		ip6tables -A FORWARD_PREFIX ! -s $FULLPREFIX -i br0 -j REJECT --reject-with icmp6-dst-unreachable
		ip6tables -D FORWARD_FIREWALL ! -i br+ -j DROP
		ip6tables -A FORWARD_FIREWALL -d $FULLLANPREFIX -i gre+ -o br0 -j ACCEPT
		ip6tables -A FORWARD_FIREWALL -d $FULLLANPREFIX -i ppp256 -o br0 -j ACCEPT
		ip6tables -A FORWARD_FIREWALL ! -i br+ -j DROP

		echo "ip6tables -t mangle -D PRE_LAN_SUBNET -s $FULLLANPREFIX -i br0 -j ACCEPT" >> /opt/var/lib/dibbler/undo_ipv6_fw.sh
		echo "ip6tables -t mangle -D ROUTE_CTL_LIST -d $FULLLANPREFIX -j RETURN" >> /opt/var/lib/dibbler/undo_ipv6_fw.sh
		echo "ip6tables -D FORWARD_FIREWALL -d $FULLLANPREFIX -i gre+ -o br0 -j ACCEPT" >> /opt/var/lib/dibbler/undo_ipv6_fw.sh
		echo "ip6tables -D FORWARD_FIREWALL -d $FULLLANPREFIX -i ppp256 -o br0 -j ACCEPT" >> /opt/var/lib/dibbler/undo_ipv6_fw.sh
		if [ "$LOG" == "1" ]
		then
			echo 'DHCP6Update: IPv6 /60 FW prepared' >> $LOGFILE
		fi
	fi
	
	if [ -n "'ip -6 route | grep "$FULLLANPREFIX"'" ]
	then
		if [ "$LOG" == "1" ]
		then 
			echo "IPv6 $FULLLANPREFIX route ok" >> $LOGFILE
		fi
	 else
		# clean up routing and prepare
		if [ -f /opt/var/lib/dibbler/undo_ipv6_route.sh ]
		then
			/opt/var/lib/dibbler/undo_ipv6_route.sh
  			if [ "$LOG" == "1" ]
			then
  				echo "DHCP6Update: IPv6 routing cleanup done" >> $LOGFILE
  			fi
			# make new routing setup                                                                                               
        		echo > /opt/var/lib/dibbler/undo_ipv6_route.sh                                                                         
        		chmod -x /opt/var/lib/dibbler/undo_ipv6_route.sh                                                                       
        		if [ "$LOG" == "1" ]
			then                                                                                             
                		echo "DHCP6Update: IPv6 routing prepared" >> $LOGFILE                                                          
        		fi 
  		fi
	fi

	#	
	# the /64-routing setup from the pools realized via /opt/bin/dibble-server-notify.sh	
	#
	OLDMD5SUM="$( md5sum /opt/etc/dibbler/server.conf)"
	sed "s~pd-pool .*~pd-pool $FULLLANPREFIX~" -i /opt/etc/dibbler/server.conf
	NEWMD5SUM="$( md5sum /opt/etc/dibbler/server.conf)"
	if [ "$OLDMD5SUM" != "$NEWMD5SUM" ]
	then
		/opt/sbin/dibbler-server stop
		if [ "$LOG" == "1" ]
		then
			echo "DHCP6Update: DHCPv6 is updated" >> $LOGFILE
		fi
	else
		if [ "$LOG" == "1" ]
		then
			echo "DHCP6Update: DHCPv6 is OK" >> $LOGFILE
		fi
	fi
	/opt/sbin/dibbler-server status || /opt/sbin/dibbler-server start
	if [ "$LOG" == "1" ]
	then
		echo "DHCP6Update: DHCPv6 is Running_Started" >> $LOGFILE
	fi
fi
